#ifndef _ASF_H
#define _ASF_H

#define ERROR -1
#define BUFSIZE 65536
 
#define MAX_SEQNUMS 16

// used by asfrecorder
#define HDR_TOTAL_SIZE_8               0x28
#define HDR_NUM_PACKETS_8              0x38
#define HDR_FINE_TOTALTIME_8           0x40
#define HDR_FINE_PLAYTIME_8            0x48
#define HDR_PLAYTIME_OFFSET_4          0x50
#define HDR_FLAGS_4                    0x58
#define HDR_ASF_CHUNKLENGTH_4          0x5c
#define HDR_ASF_CHUNKLENGTH_CONFIRM_4  0x60

#define DATSEG_HDR_SIZE    50

#define DATSEG_SEGSIZE_8   0x10
#define DATSEG_NUMCHUNKS_4 0x28

struct asf_file
{
	FILE *file;
	struct asf_header *asfh;
	struct asf_file_header *asffh;
	struct asf_stream_header *asfsh;
	struct asf_data_header *asfdh;
};

struct asf_packet_header
{
  unsigned short type;
  unsigned short length;
  //length is everything from this point on
  // i.e., length = sizeof(struct asf_packet_header) + total_data_len - 4;
  unsigned long seqnum;
  unsigned short flags;
  unsigned short length_2;
};

struct asf_header
{
  unsigned char id[16]; // guid
  unsigned long size_lo, size_hi;
	unsigned long unknown_1; // 0x06
	unsigned char unknown_2[2]; // 0x01 0x02 
	unsigned char unknown_id[16]; // ce 75 f8 7b 8d 46 d1 11 8d 82 00 60 97 c9 a2 b2
	unsigned long unknown_size_lo, unknown_size_hi;
};

struct asf_file_header
{
  unsigned char id[16]; // guid
  unsigned long size_lo, size_hi;
  unsigned char file_id[16];
  unsigned long file_size_lo, file_size_hi;
  unsigned long file_time_lo, file_time_hi;
  unsigned long unknown_1_lo, unknown_1_hi; // 0x0283
  unsigned long unknown_2_lo, unknown_2_hi; // asfencoder labels this stream 0
  unsigned long unknown_3_lo, unknown_3_hi; // asfencoder labels this stream 1
  unsigned long unknown_4; // 0x0c1c
  unsigned long unknown_5; // 0x00
  unsigned long unknown_6; // 0x02
  unsigned long chunklength, chunklength_2;
  unsigned long unknown_7; // 0x03e800
};

typedef struct _WAVEFORMATEX
{
	unsigned short format_type;      // format type
	unsigned short num_channels;     // number of channels (i.e., mono, stereo, ...)
	unsigned long samples_per_sec;   // sample rate
	unsigned long avg_bytes_per_sec; // for buffer estimation
	unsigned short block_size;       // block size of data
	unsigned short bits_per_sample;  // # of bits per sample of mono data
	unsigned short size;             // the count in bytes of the size of extra information (after size)
} WAVEFORMATEX;

#if 0
typedef struct _BITMAPHEADERINFO
{
	unsigned long size;
	long width;
	long height;
	unsigned short planes;
	unsigned short bit_count;
	unsigned long compression;
	unsigned long image_size;
	long x_pels_per_meter;
	long y_pels_per_meter;
	unsigned long clr_used;
	//unsigned long clr_important; // TODO: too big?
} BITMAPHEADERINFO;
#endif

struct asf_stream_header
{
  unsigned char id[16]; // guid
  unsigned long size_lo, size_hi;
  unsigned char type[16]; // guid
  unsigned char method[16]; // guid
  unsigned long unknown_1_lo, unknown_1_hi;
	unsigned long datalen_1, datalen_2;
	unsigned short stream_id;
	unsigned long unknown_2;

	union
	{
		WAVEFORMATEX audio;
#if 0	
		struct _video
		{
			unsigned long width;
			unsigned long height;
			unsigned char unknown;
			unsigned short size;
			BITMAPHEADERINFO bitmap;
		} video;
#endif
	} data;
};
 
struct asf_data_header
{
	unsigned char id[16];
	unsigned long size_lo, size_hi;
	unsigned char packet_id[16]; // guid_packet
	unsigned long packet_size_lo, packet_size_hi;
};

/*
struct asf_data_header // TODO: this should be 50, not 60 bytes
{
	unsigned long total_size_lo, total_size_hi;
  unsigned char unknown_1[8];
  unsigned long numpackets_lo, numpackets_hi;
  unsigned long total_time_lo, total_time_hi;
  unsigned long play_time_lo, play_time_hi;
  unsigned long playtime_offset;
  unsigned char unknown_2[4];
  unsigned long flags;
  unsigned long chunklength, chunklength_2;
};
*/

struct asf_frame_header
{
	unsigned char value; // stream_num | 0x80 
	unsigned char seqnum;
	unsigned long frag_offset;
	unsigned char flags; // 0x08
	unsigned long data_length;
	unsigned long timestamp;
	unsigned short frag_length;
};

struct asf_content_type
{
	unsigned char id[16];
	unsigned long size_lo, size_hi;
	unsigned char unknown_id[16];
	unsigned long unknown;
};

//////////////////////////////////////////////////////////////
// Function declarations

extern int asf_create_file(char *filename);
extern int asf_append_file(char *filename);
extern int asf_read_file(char *filename);
extern int asf_read_asx_file(char *filename, char *data, int size);

#if 1
extern void asf_close_file(int fd);
extern int asf_reset_file(int fd);

extern int asf_write_header(int fd, struct asf_header *asfh, int size);
extern int asf_write_data(int fd, unsigned char *data, int size, unsigned long seqnum, unsigned long chunklength);

extern struct asf_header *asf_locate_header(int fd);
extern struct asf_file_header *asf_locate_file_header(int fd);
extern struct asf_stream_header *asf_locate_stream_header(int fd);
extern struct asf_data_header *asf_locate_data_header(int fd);
#else
extern void asf_close_file(struct asf_file *file);
extern int asf_reset_file(struct asf_file *file);
 
extern int asf_write_header(struct asf_file *file, struct asf_header *asfh, int size);
extern int asf_write_data(struct asf_file *file, unsigned char *data, int size, unsigned long seqnum, unsigned long chunklength);
 
extern struct asf_header *asf_locate_header(struct asf_file *file);
extern struct asf_file_header *asf_locate_file_header(struct asf_file *file);
extern struct asf_stream_header *asf_locate_stream_header(struct asf_file *file);
extern struct asf_data_header *asf_locate_data_header(struct asf_file *file);
#endif

extern void asf_print_packet_header(struct asf_packet_header *asfph);
extern void asf_print_header(struct asf_header *asfh);
extern void asf_print_file_header(struct asf_file_header *asffh);
extern void asf_print_stream_header(struct asf_stream_header *asfsh);
extern void asf_print_data_header(struct asf_data_header *asfdh);

extern unsigned long asf_get_timecode(unsigned char *buf);
extern int asf_fix_timecodes(unsigned char *buf, unsigned long deltatime, unsigned long chunknum, unsigned long chunklength); 
#endif
