#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>

#include "guids.h"
#include "misc.h"
#include "asf.h"

int main(int argc, char **argv)
{
#if 1
	int fd;
#else
	struct asf_file *file;
#endif
	
  int found_stream = 0;

	register int i;
	unsigned long chunklength;
	
	struct asf_header *asfh;
	struct asf_file_header *asffh;
	struct asf_stream_header *asfsh;
	struct asf_data_header *asfdh;

  if (argc < 2)
	{
		fprintf(stderr, "Usage: %s <ASF file> [ASF file 2] [...]\n", argv[0]);
		return -1;
	}

	for (i = 1; i < argc; i++)
	{
		printf("Reading \"%s\"\n\n", argv[i]);
#if 1
		fd = asf_read_file(argv[i]);
		if (fd < 0) return -1;
#else
		file = asf_read_file(argv[1]);
		if (!file) return -1;
#endif
		
		// HEADER
#if 1
		asfh = asf_locate_header(fd);
#else
		asfh = asf_locate_header(file);
#endif
		if (!asfh)
		{
			printf("Error locating header\n");
			goto closeup;
		}
		
		asf_print_header(asfh);
		
		// FILE HEADER
#if 1
		asffh = asf_locate_file_header(fd);
#else
		asffh = asf_locate_file_header(file);
#endif
		if (!asffh)
		{
			printf("Error locating file header\n");
			goto closeup;
		}
		
		chunklength = asffh->chunklength;
		asf_print_file_header(asffh);

#if 0 // TODO: why doesn't this work?
		{
			unsigned long hi, lo;
			unsigned char buf[8];
			int numbytes;
			
			printf("lseek HDR_TOTAL_SIZE_8 (offset %u)\n", HDR_TOTAL_SIZE_8);
			if (lseek(fd, HDR_TOTAL_SIZE_8, SEEK_SET) < 0) return -1;
			numbytes = read(fd, buf, 8);
			if (numbytes != 8) return -1;
			get_quad(buf, &hi, &lo);
		
			printf("total size = 0x%08lx, 0x%08lx (%lu, %lu)\n\n", hi, lo, hi, lo);
		
			printf("lseek HDR_NUM_PACKETS_8 (offset %u)\n", HDR_NUM_PACKETS_8);
			if (lseek(fd, HDR_NUM_PACKETS_8, SEEK_SET) < 0) return -1;
			numbytes = read(fd, buf, 8);
			if (numbytes != 8) return -1;
			get_quad(buf, &hi, &lo);
			
			printf("numpackets = 0x%08lx, 0x%08lx (%lu, %lu)\n\n", hi, lo, hi, lo);
			
			printf("lseek HDR_FINE_TOTALTIME_8 (offset %u)\n", HDR_FINE_TOTALTIME_8);
			if (lseek(fd, HDR_FINE_TOTALTIME_8, SEEK_SET) < 0) return -1;
			numbytes = read(fd, buf, 8);
			if (numbytes != 8) return -1;
			get_quad(buf, &hi, &lo);
			
			printf("total time = 0x%08lx, 0x%08lx (%lu, %lu)\n\n", hi, lo, hi, lo);

			printf("lseek HDR_FINE_PLAYTIME_8 (offset %u)\n", HDR_FINE_PLAYTIME_8);
			if (lseek(fd, HDR_FINE_PLAYTIME_8, SEEK_SET) < 0) return -1;
			numbytes = read(fd, buf, 8);
			if (numbytes != 8) return -1;
			get_quad(buf, &hi, &lo);
			
			printf("playtime = 0x%08lx, 0x%08lx (%lu, %lu)\n\n", hi, lo, hi, lo);
			
			printf("lseek HDR_PLAYTIME_OFFSET_4 (offset %u)\n", HDR_PLAYTIME_OFFSET_4);
			if (lseek(fd, HDR_PLAYTIME_OFFSET_4, SEEK_SET) < 0) return -1;
			numbytes = read(fd, buf, 4);
			if (numbytes != 4) return -1;
			get_long(buf, &lo);
			
			printf("playtime offset = 0x%08lx (%lu)\n\n", lo, lo);

			printf("lseek HDR_FLAGS_4 (offset %u)\n", HDR_FLAGS_4);
			if (lseek(fd, HDR_FLAGS_4, SEEK_SET) < 0) return -1;
			numbytes = read(fd, buf, 4);
			if (numbytes != 4) return -1;
			get_long(buf, &lo);
		
			printf("flags = 0x%08lx (%lu)\n\n", lo, lo);
		}
		
		{	
			unsigned long hi, lo;
			unsigned char buf[8];
			int numbytes;
			int s_header_size = asffh->size_hi + asffh->size_lo;
			int offset = s_header_size - DATSEG_HDR_SIZE;
			
			printf("lseek offset (s_header_size - DATSEG_HDR_SIZE) + DATSEG_SEGSIZE_8 (offset %u + %u)\n", offset, DATSEG_SEGSIZE_8);
			if (lseek(fd, offset + DATSEG_SEGSIZE_8, SEEK_SET) < 0) return -1;
			numbytes = read(fd, buf, 8);
			if (numbytes != 8) return -1;
			get_quad(buf, &hi, &lo);
		
			printf("segsize = 0x%08lx, 0x%08lx (%lu, %lu)\n\n", hi, lo, hi, lo);
			
			printf("lseek offset (s_header_size - DATSEG_HDR_SIZE) + DATSEG_NUMCHUNKS_4 (offset %u + %u)\n", offset, DATSEG_NUMCHUNKS_4);
			if (lseek(fd, offset + DATSEG_NUMCHUNKS_4, SEEK_SET) < 0) return -1;
			numbytes = read(fd, buf, 4);
			if (numbytes != 4) return -1;
			get_quad(buf, &hi, &lo);
			
			printf("numchunks = 0x%08lx (%lu)\n\n", lo, lo);
		}

#if 0
		if (!asfdh->chunklength || !asfdh->chunklength_2)
		{
			printf("Erorr: corrupt data header (chunklength = 0x%08lx, chunklength_2 = 0x%08lx)\n\n", asfdh->chunklength, asfdh->chunklength_2);
			return;
		}
	 
		if (asfdh->chunklength != asfdh->chunklength_2)
		{
			printf("Error: corrupt data header (chunklength (0x%08lx) != chunklength_2 (0x%08lx))\n\n", asfdh->chunklength, asfdh->chunklength_2);
			return;
		}
	 
		printf("total_size 0x%08lx 0x%08lx (%lu)\n", asfdh->total_size_hi, asfdh->total_size_lo);
	 
		printf("unknown_1[] = ");
		print_hex(asfdh->unknown_1, 8, -1);
		putchar('\n');
	 
		printf("numpackets = 0x%08lx 0x%08lx (%lu)\n", asfdh->numpackets_hi, asfdh->numpackets_lo, asfdh->numpackets_lo);
		printf("total_time = 0x%08lx 0x%08lx (%lu)\n", asfdh->total_time_hi, asfdh->total_time_lo, asfdh->total_time_lo);
		printf("play_time = 0x%08lx 0x%08lx (%lu)\n", asfdh->play_time_hi, asfdh->play_time_lo, asfdh->play_time_lo);
		printf("playtime_offset = 0x%08lx (%lu)\n", asfdh->playtime_offset, asfdh->playtime_offset);
	 
		printf("unknown_2[] = ");
		print_hex(asfdh->unknown_2, 4, -1);
		putchar('\n');
	 
		printf("flags = 0x%08lx\n", asfdh->flags);
		printf("chunklength = 0x%08lx, chunklength_2 = 0x%08lx\n", asfdh->chunklength - sizeof(struct asf_data_chunk_header),
					 asfdh->chunklength_2 - sizeof(struct asf_data_chunk_header));
#endif

		goto closeup;
#endif
		// DATA HEADER
#if 1
		asfdh = asf_locate_data_header(fd);
#else
		asfdh = asf_locate_data_header(file);
#endif
		if (!asfdh)
		{
			printf("Error locating data header\n");
			goto closeup;
		}
		
		// STREAM HEADERS
		while (1)
		{
#if 1
			asfsh = asf_locate_stream_header(fd);
#else
			asfsh = asf_locate_stream_header(file);
#endif
			if (!asfsh) break;

			found_stream = 1;
			
			asf_print_stream_header(asfsh);
			putchar('\n');
		}

		if (!found_stream)
		{
			printf("Error locating stream headers\n");
			goto closeup;
		}
	
closeup:
		printf("Closing \"%s\"\n", argv[i]);
#if 1
		asf_close_file(fd);
#else
		asf_close_file(file);
#endif
	}

	return 0;
}
