#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>
#include <ctype.h>
 
#include "asf.h"
#include "guids.h"

unsigned char guid_header[16] = { 0x30, 0x26, 0xb2, 0x75, 0x8e, 0x66,
0xcf, 0x11, 0xa6, 0xd9, 0x00, 0xaa, 0x00, 0x62, 0xce, 0x6c };

unsigned char guid_header2[16] = { 0xd1, 0x29, 0xe2, 0xd6, 0xda,
0x35, 0xd1, 0x11, 0x90, 0x34, 0x00, 0xa0, 0xc9, 0x03, 0x49, 0xbe };

unsigned char guid_file_header[16] = { 0xa1, 0xdc, 0xab, 0x8c, 0x47, 0xa9, 
0xcf, 0x11, 0x8e, 0xe4, 0x00, 0xc0, 0x0c, 0x20, 0x53, 0x65 };

unsigned char guid_stream_header[16] = { 0x91, 0x07, 0xdc, 0xb7, 0xb7,
0xa9, 0xcf, 0x11, 0x8e, 0xe6, 0x00, 0xc0, 0x0c, 0x20, 0x53, 0x65 };


unsigned char guid_data_chunk[16] = { 0x36, 0x26, 0xb2, 0x75, 0x8e, 0x66,
0xcf, 0x11, 0xa6, 0xd9, 0x00, 0xaa, 0x00, 0x62, 0xce, 0x6c };


unsigned char guid_content_type[16] = { 0x40, 0x52, 0xd1, 0x86, 0x1d,
0x31, 0xd0, 0x11, 0xa3, 0xa4, 0x00, 0xa0, 0xc9, 0x03, 0x48, 0xf6 };


unsigned char guid_url[16] = { 0xe3, 0xcb, 0x1a, 0x4b, 0x0b, 0x10, 0xd0,
0x11, 0xa3, 0x9b, 0x00, 0xa0, 0xc9, 0x03, 0x48, 0xf6 };


// Stream stuff
unsigned char guid_audio_stream[16] = { 0x40, 0x9e, 0x69, 0xf8, 0x4d,
0x5b, 0xcf, 0x11, 0xa8, 0xfd, 0x00, 0x80, 0x5f, 0x5c, 0x44, 0x2b };

unsigned char guid_video_stream[16] = { 0xc0, 0xef, 0x19, 0xbc, 0x4d,
0x5b, 0xcf, 0x11, 0xa8, 0xfd, 0x00, 0x80, 0x5f, 0x5c, 0x44, 0x2b };

// scripting stuff?
unsigned char guid_unknown_stream[16] = { 0x00, 0xe1, 0x1b, 0xb6, 0x4e, 0x5b, 
0xcf, 0x11, 0xa8, 0xfd, 0x00, 0x80, 0x5f, 0x5c, 0x44, 0x2b };

unsigned char guid_audio_conceal_none[16] = { 0x40, 0xa4, 0xf1, 0x49,
0xce, 0x4e, 0xd0, 0x11, 0xa3, 0xac, 0x00, 0xa0, 0xc9, 0x03, 0x48, 0xf6};

unsigned char guid_audio_conceal_interleaved[16] = { 0x50, 0xcd, 0xc3,
0xbf, 0x8f, 0x61, 0xcf, 0x11, 0x8b, 0xb2, 0x00, 0xaa, 0x00, 0xb4, 0xe2,
0x20 };

unsigned char guid_video_conceal_none[16] = { 0x00, 0x57, 0xfb, 0x20, 
0x55, 0x5b, 0xcf, 0x11, 0xa8, 0xfd, 0x00, 0x80, 0x5f, 0x5c, 0x44, 0x2b };


// Bitrate stuff
unsigned char guid_bitrates_1[16] = { 0xce, 0x75, 0xf8, 0x7b, 0x8d, 0x46,
0xd1, 0x11, 0x8d, 0x82, 0x00, 0x60, 0x97, 0xc9, 0xa2, 0xb2 };

unsigned char guid_bitrates_2[16] = { 0x01, 0x2a, 0xe2, 0xd6, 0xda, 0x35, 0xd1, 
0x11, 0x90, 0x34, 0x00, 0xa0, 0xc9, 0x03, 0x49, 0xbe };


// Unknown
unsigned char guid_unknown_1[16] = { 0xb5, 0x03, 0xbf, 0x5f, 0x2e, 0xa9,
0xcf, 0x11, 0x8e, 0xe3, 0x00, 0xc0, 0x0c, 0x20, 0x53, 0x65 };

unsigned char guid_unknown_2[16] = { 0x11, 0xd2, 0xd3, 0xab, 0xba, 0xa9,
0xcf, 0x11, 0x8e, 0xe6, 0x00, 0xc0, 0x0c, 0x20, 0x53, 0x65 };


unsigned char guid_unknown_3[16] = { 0x30, 0x1a, 0xfb, 0x1e, 0x62, 0x0b, 
0xd0, 0x11, 0xa3, 0x9b, 0x00, 0xa0, 0xc9, 0x03, 0x48, 0xf6 };

