// Auxiliary functions

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <stdarg.h>
#include "util/util_all.h"
#include "net/net_all.h"
#include "misc.h"

/*
 * find_guid()
 */
unsigned char *find_guid(unsigned char *guid, unsigned char *buf, unsigned short length)
{
  register unsigned short i, j;

  if (!guid || !buf || length < 16) return NULL;
  
  for (i = 0; i <= length - 16; ++i)
  {
    for (j = 0; j < 16; ++j)
		{
      if (buf[i + j] != guid[j]) break;
		}

    if (j == 16) return buf + i;
  }

  return NULL;  
}

/*
 * print_time
 */
void print_time(unsigned long timecode)
{
  if ((timecode / 1000) >= (24 * 3600)) {
    printf("%ld:%02ld:%02ld:%02ld.%03ld", (timecode / 1000) / (24 * 3600), ((timecode / 1000) / 3600)%24, 
           ((timecode / 1000) / 60) % 60, (timecode / 1000) % 60, timecode % 1000);
  } 
  else {
    if ((timecode / 1000) >= 3600) {
      printf("%ld:%02ld:%02ld.%03ld", (timecode / 1000) / 3600, ((timecode / 1000) / 60) % 60, (timecode / 1000) % 60, timecode % 1000);
    }
    else { 
      if ((timecode / 1000) >= 60) printf("%ld:%02ld.%03ld", (timecode / 1000) / 60, (timecode / 1000) % 60, timecode % 1000);
      else printf("%ld.%03ld", (timecode / 1000) % 60, timecode % 1000);
    }
  }
}

/*
 * print_hex()
 */
void print_hex(unsigned char *str, unsigned short length, int index)
{
  register unsigned short i;

  if (!str || !length) return;
  if (index >= 0) printf("  [0x%04x] ", index);

  for (i = 1; i <= length; i++)
  {
    printf("%02x ", str[i-1]);

    if (index >= 0) index++;
    if (!(i % 16) && i != length)
		{
			if (index >= 0) printf("\n  [0x%04x] ", index);
  		else putchar('\n');
		}
	}

  putchar('\n');
}

///////////////////////////////////////////////////
// asfrecorder functions

/*
 * base64enc
 */ 
const char alpha[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=";
 
char *base64enc(char *fmt, ...)
{
    static char inbuf[BUFFERSIZE], outbuf[BUFFERSIZE];
	  int len;
    register int i, index, val;
 
		va_list ap;
		va_start(ap, fmt);
		vsnprintf(inbuf, sizeof(inbuf), fmt, ap);
		va_end(ap);
		
	  len = strlen(inbuf);
    
		for (i=0, index=0; i<len; i+=3, index+=4)
    {
        register int quad, trip;
 
        quad = trip = 0;
 
        val = (0xff & inbuf[i]) << 8;
        
				if ((i+1) < len)
				{
            val |= (0xff & inbuf[i+1]);
            trip = 1;
        }
				
        val <<= 8;
        
				if ((i+2) < len)
				{
            val |= (0xff & inbuf[i+2]);
            quad = 1;
        }
        outbuf[index+3] = alpha[(quad? (val & 0x3f): 64)], val >>= 6;
        outbuf[index+2] = alpha[(trip? (val & 0x3f): 64)], val >>= 6;
        outbuf[index+1] = alpha[val & 0x3f], val >>= 6;
        outbuf[index+0] = alpha[val & 0x3f];
    }
 
    outbuf[++index] = 0;
    return outbuf;
} 

unsigned long get_data(unsigned char **bp, unsigned int type)
{
    unsigned long retval = 0;
		
    switch (type)
		{
        case 0:
					break;
        
				case 1:
					retval = *(*bp+0);
					*bp += 1;
					break;
					
        case 2:
					retval = *(*bp+0) + (*(*bp+1) << 8);
					*bp += 2;
					break;
        
				case 3:
					retval = *(*bp+0) + (*(*bp+1) << 8) + (*(*bp+2) << 16) + (*(*bp+3) << 24);
					*bp += 4;
					break;
				
				default:
					printf("Error: unknown format type 0x%02x\n", type);
					break;
    }
    return retval;
} 

/*
 * get_quad
 */
void get_quad(unsigned char *pos, unsigned long *hi, unsigned long *lo)
{
    unsigned char c1 = *pos++, c2 = *pos++, c3 = *pos++, c4 = *pos++;
    unsigned char c5 = *pos++, c6 = *pos++, c7 = *pos++, c8 = *pos++;
		(*hi) = (c8<<24) + (c7<<16) + (c6<<8) + c5;
    (*lo) = (c4<<24) + (c3<<16) + (c2<<8) + c1;
}

/*
 * write_quad
 */
int write_quad(int fd, unsigned long hi, unsigned long lo)
{
    unsigned char c1 = (lo & 0xff), c2 = ((lo>>8) & 0xff), c3 = ((lo>>16) & 0xff), c4 = ((lo>>24) & 0xff);
    unsigned char c5 = (hi & 0xff), c6 = ((hi>>8) & 0xff), c7 = ((hi>>16) & 0xff), c8 = ((hi>>24) & 0xff);
#if 0
		write(fd, &c1, 1); write(fd, &c2, 1); write(fd, &c3, 1); write(fd, &c4, 1);
    write(fd, &c5, 1); write(fd, &c6, 1); write(fd, &c7, 1); write(fd, &c8, 1);
#else
		int numbytes;
		register int i = 0;
		unsigned char buf[8];
		
		buf[i++] = c1, buf[i++] = c2, buf[i++] = c3, buf[i++] = c4, buf[i++] = c5, buf[i++] = c6, buf[i++] = c7, buf[i++] = c8;
		numbytes = write(fd, buf, 8);
		if (numbytes != 8)
		{
			fprintf(stderr, "Error writing quad (wrote %d of 8 bytes): %s\n", numbytes, strerror(errno));
			return -1;
		}

		return 0;
#endif
}

/*
 * get_long
 */
void get_long(unsigned char *pos, unsigned long *val)
{
    unsigned char c1 = *pos++, c2 = *pos++, c3 = *pos++, c4 = *pos++;
    (*val) = (c4<<24) + (c3<<16) + (c2<<8) + c1;
}

/*
 * put_long
 */
void put_long(unsigned char *pos, unsigned long val)
{
    *pos++ = (val & 0xff); *pos++ = ((val>>8) & 0xff), *pos++ = ((val>>16) & 0xff), *pos++ = ((val>>24) & 0xff);
}

/*
 * write_long
 */
int write_long(int fd, unsigned long val)
{
    unsigned char c1 = (val & 0xff), c2 = ((val>>8) & 0xff), c3 = ((val>>16) & 0xff), c4 = ((val>>24) & 0xff);
#if 0
		write(fd, &c1, 1); write(fd, &c2, 1); write(fd, &c3, 1); write(fd, &c4, 1);
#else
		int numbytes;
		register int i = 0;
		unsigned char buf[4];
		
		buf[i++] = c1, buf[i++] = c2, buf[i++] = c3, buf[i++] = c4;
		numbytes = write(fd, buf, 4);
		if (numbytes != 4)
		{
			fprintf(stderr, "Error writing long (wrote %d of 4 bytes): %s\n", numbytes, strerror(errno));
			return -1;
		}

		return 0;
#endif
}

/*
 * get_short
 */
void get_short(unsigned char *pos, unsigned short *val)
{
    unsigned char c1 = *pos++, c2 = *pos++;
    (*val) = (c2<<8) + c1;
}

/*
 * write_short
 */
int write_short(int fd, unsigned short val)
{
    unsigned char c1 = (val & 0xff), c2 = ((val>>8) & 0xff);
#if 0
		write(fd, &c1, 1); write(fd, &c2, 1);
#else
		int numbytes;
		register int i = 0;
		unsigned char buf[2];
		
		buf[i++] = c1, buf[i++] = c2; 
		numbytes = write(fd, buf, 2);
		if (numbytes != 2)
		{
			fprintf(stderr, "Error writing short (wrote %d of 2 bytes): %s\n", numbytes, strerror(errno));
			return -1;
		}

		return 0;
#endif
}

/*
 * quad_subtraction
 */
void quad_subtraction(unsigned long *dst_hi, unsigned long *dst_lo, unsigned long hi, unsigned long lo)
{
    unsigned long tmp;

    *dst_hi -= hi;

    tmp = *dst_lo;
    *dst_lo -= lo;

    /* handle "carry bit" */
    if (*dst_lo > tmp) *dst_hi -= 1;
}
